@@ -942,6 +942,9 @@ bool GR_Win32Graphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 		DEVMODE *pDevMode = (DEVMODE*) GlobalLock(m_hDevMode);
 		pDevMode->dmFields = DM_ORIENTATION;
 		pDevMode->dmOrientation = (bPortrait) ? DMORIENT_PORTRAIT : DMORIENT_LANDSCAPE;
+		GlobalUnlock(m_hDevMode);
+		
+		pDevMode = (DEVMODE*) GlobalLock(m_hDevMode);
 		ResetDC(m_hdc, pDevMode);
 		GlobalUnlock(m_hDevMode);
 	}
@@ -2185,3 +2188,75 @@ GR_Graphics * GR_Win32Graphics::getPrinterGraphics(const char * pPrinterName,
 	return pGr;
 }
 
+
+/*!
+    This function absolutely must be called anytime the DEVMODE structure is modified in
+    any way; it also must be called on the DEVMODE handle returned by PringDlg. Those who
+    will modify DEVMODE structure and not call this function on it will be shot, without
+    exception.
+
+    The issue with the DEVMODE structure is this: it consists of two parts, a public part
+    declared by the win32 headers, and a private part, the size and contents of which 
+    depend on the printer. Before a printer can be passed the DEVMODE structure, two
+    things have to be satisfied:
+
+    1. The memory allocated for the structure has to be large enough to hold both the
+       public and private parts of the structure; we obtain the handle from the
+       PrintDlg, and it _should_ allocate the necessary size correctly. In any case, we
+       cannot meddle with the size, since reallocating means loosing the private part.
+
+    2. The settings represented by the public part have to be transferred into the private
+       part; this is achieved by calling the DocumentProperties() function on the
+       structure. So any time we modify the contents of DEVMODE in any way we have to call
+       DocumentProperties() before we can pass the pointer / handle to any other
+       function. In addition, I have a strong suspicion that the PrintDlg function only
+       sets the public section, but does not call DocumentProperties(), so we have to do
+       that ourselves once we get the handle back.
+*/
+bool GR_Win32Graphics::fixDevMode(HGLOBAL hDevMode)
+{
+	DEVMODE * pDM = (DEVMODE*)GlobalLock(hDevMode);
+	UT_return_val_if_fail( pDM, false );
+	
+	// find out how big structure this particular printer really needs
+	HANDLE      hPrinter;
+	DWORD       dwNeeded, dwRet;
+
+	//Start by opening the printer
+	if(!OpenPrinter((char*)& pDM->dmDeviceName, &hPrinter, NULL))
+	{
+		UT_String msg;
+		UT_String_sprintf(msg,"Unable to open printer [%s]", (char*)& pDM->dmDeviceName);
+		LOG_WIN32_EXCPT(msg.c_str());
+		GlobalUnlock(hDevMode);
+		UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+		return false;
+	}
+	
+	dwNeeded = DocumentProperties(NULL,hPrinter, (char*)& pDM->dmDeviceName, NULL, NULL, 0);
+
+	if( (int) dwNeeded > pDM->dmSize + pDM->dmDriverExtra )
+	{
+		// we are knees deep in crap here ... log it into the users profile, so we have
+		// some debug info to go on
+		UT_String msg;
+		UT_String_sprintf(msg,"DEVMODE too small [%s, needs %d, got %d + %d]",
+						  (char*)& pDM->dmDeviceName, dwNeeded, pDM->dmSize, pDM->dmDriverExtra);
+		LOG_WIN32_EXCPT(msg.c_str());
+		ClosePrinter(hPrinter);
+		GlobalUnlock(hDevMode);
+		UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+		return false;
+	}
+	
+	// now get the printer driver to merge the data in the public section into its private part
+	dwRet = DocumentProperties(NULL,hPrinter, (char*)& pDM->dmDeviceName, pDM, pDM, DM_IN_BUFFER | DM_OUT_BUFFER);
+
+	// free what needs be ...
+	ClosePrinter(hPrinter);
+	GlobalUnlock(hDevMode);
+
+	UT_return_val_if_fail( dwRet == IDOK, false );
+	return true;
+}
+
