@@ -29,6 +29,7 @@
 #include "xap_CocoaApp.h"
 #include "xap_CocoaAppController.h"
 #include "xap_CocoaModule.h"
+#include "xap_CocoaPlugin.h"
 #include "xap_CocoaToolPalette.h"
 #include "xap_App.h"
 #include "xap_Frame.h"
@@ -307,23 +308,70 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 		{
 			XAP_AppController_Instance = self;
 
-			m_bFileOpenedDuringLaunch = NO;
+			m_FilesRequestedDuringLaunch = [[NSMutableArray alloc] initWithCapacity:8];
+
 			m_bApplicationLaunching   = YES;
 
 			m_bAutoLoadPluginsAfterLaunch = NO;
 
+			m_PanelMenu   = [[NSMenu alloc] initWithTitle:@"Panels"];
 			m_ContextMenu = [[NSMenu alloc] initWithTitle:@"Context Menu"];
+
+			m_FontDictionary = 0;
+
+			m_MenuIDRefDictionary = [[NSMutableDictionary alloc] initWithCapacity:16];
+
+			m_Plugins      = [[NSMutableArray alloc] initWithCapacity:16];
+			m_PluginsTools = [[NSMutableArray alloc] initWithCapacity:16];
+
+			m_PluginsToolsSeparator = [NSMenuItem separatorItem];
+			[m_PluginsToolsSeparator retain];
 		}
 	return self;
 }
 
 - (void)dealloc
 {
+	if (m_FilesRequestedDuringLaunch)
+		{
+			[m_FilesRequestedDuringLaunch release];
+			m_FilesRequestedDuringLaunch = 0;
+		}
+	if (m_PanelMenu)
+		{
+			[m_PanelMenu release];
+			m_PanelMenu = 0;
+		}
 	if (m_ContextMenu)
 		{
 			[m_ContextMenu release];
 			m_ContextMenu = 0;
 		}
+	if (m_FontDictionary)
+		{
+			[m_FontDictionary release];
+			m_FontDictionary = 0;
+		}
+	if (m_MenuIDRefDictionary)
+		{
+			[m_MenuIDRefDictionary release];
+			m_MenuIDRefDictionary = 0;
+		}
+	if (m_Plugins)
+		{
+			[m_Plugins release];
+			m_Plugins = 0;
+		}
+	if (m_PluginsTools)
+		{
+			[m_PluginsTools release];
+			m_PluginsTools = 0;
+		}
+	if (m_PluginsToolsSeparator)
+		{
+			[m_PluginsToolsSeparator release];
+			m_PluginsToolsSeparator = 0;
+		}
 	[super dealloc];
 }
 
@@ -345,6 +393,13 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 
 			[[NSFileManager defaultManager] changeCurrentDirectoryPath:desktop];
 		}
+	if (NSMenu * menu = [NSApp windowsMenu])
+		if (NSMenuItem * item = [[NSMenuItem alloc] initWithTitle:@"Panels" action:nil keyEquivalent:@""])
+			{
+				[menu addItem:item];
+				[item setSubmenu:m_PanelMenu];
+				[item release];
+			}
 }
 
 - (void)applicationDidFinishLaunching:(NSNotification *)aNotification
@@ -386,7 +441,35 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 			UT_DEBUGMSG(("[...FinishLaunching] Auto-Loading plug-ins:\n"));
 			XAP_CocoaModule::loadAllPlugins();
 		}
-	if (m_bFileOpenedDuringLaunch == NO)
+
+	BOOL bFileOpenedDuringLaunch = NO;
+
+	unsigned count = [m_FilesRequestedDuringLaunch count];
+
+	for (unsigned i = 0; i < count; i++) // filter out plugins from list and open those first
+		{
+			NSString * filename = (NSString *) [m_FilesRequestedDuringLaunch objectAtIndex:i];
+
+			UT_UTF8String path([filename UTF8String]);
+
+			if (XAP_CocoaModule::hasPluginExtension(path))
+				{
+					[self application:NSApp openFile:filename];
+				}
+		}
+	for (unsigned i = 0; i < count; i++) // open the rest
+		{
+			NSString * filename = (NSString *) [m_FilesRequestedDuringLaunch objectAtIndex:i];
+
+			UT_UTF8String path([filename UTF8String]);
+
+			if (!XAP_CocoaModule::hasPluginExtension(path))
+				if ([self application:NSApp openFile:filename])
+					{
+						bFileOpenedDuringLaunch = YES;
+					}
+		}
+	if (bFileOpenedDuringLaunch == NO)
 		{
 			UT_DEBUGMSG(("[...FinishLaunching] No file opened during launch, so opening untitled document:\n"));
 			[self applicationOpenUntitledFile:NSApp];
@@ -411,7 +494,21 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 
 - (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
 {
+	if (m_bApplicationLaunching == YES)
+		{
+			[m_FilesRequestedDuringLaunch addObject:filename];
+			return YES;
+		}
+
 	UT_DEBUGMSG(("Requested to open %s\n", [filename UTF8String]));
+
+	UT_UTF8String path([filename UTF8String]);
+
+	if (XAP_CocoaModule::hasPluginExtension(path))
+		{
+			return (XAP_CocoaModule::loadPlugin(path) ? YES : NO);
+		}
+
 	XAP_App * pApp = XAP_App::getApp();
 	XAP_Frame * pNewFrame = pApp->newFrame();
 
@@ -425,9 +522,6 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 	if (result)
 	{
 		pNewFrame->show();
-
-		if (m_bApplicationLaunching == YES)
-			m_bFileOpenedDuringLaunch = YES;
 	}
 	return (result ? YES : NO);
 }
@@ -452,6 +546,11 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 
 - (BOOL)applicationOpenUntitledFile:(NSApplication *)theApplication
 {
+	if (m_bApplicationLaunching == YES)
+		{
+			return YES;
+		}
+
 	UT_DEBUGMSG(("Requested to open untitled file...\n"));
 
 	EV_EditMethodContainer * pEMC = XAP_App::getApp()->getEditMethodContainer();
@@ -462,13 +561,7 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 	if (!pEM)
 		return NO;
 
-	bool result = pEM->Fn(0,0);
-	if (result)
-	{
-		if (m_bApplicationLaunching == YES)
-			m_bFileOpenedDuringLaunch = YES;
-	}
-	return (result ? YES : NO);
+	return (pEM->Fn(0,0) ? YES : NO);
 }
 
 - (BOOL)applicationOpenFile:(NSApplication *)theApplication
@@ -563,11 +656,21 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 	return equiv;
 }
 
+- (NSMenu *)panelMenu
+{
+	return m_PanelMenu;
+}
+
 - (NSMenu *)contextMenu
 {
 	return m_ContextMenu;
 }
 
+- (void)appendPanelItem:(NSMenuItem *)item
+{
+	[m_PanelMenu addItem:item];
+}
+
 - (void)appendContextItem:(NSMenuItem *)item
 {
 	[m_ContextMenu addItem:item];
@@ -661,6 +764,71 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 	[self clearMenu:XAP_CocoaAppMenu_Help  ];
 }
 
+- (NSString *)familyNameForFont:(NSString *)fontName
+{
+	NSString * familyName = nil;
+
+	if (!m_FontDictionary)
+		{
+			NSArray * pAvailableFontFamilies = [[NSFontManager sharedFontManager] availableFontFamilies];
+
+			unsigned family_count = [pAvailableFontFamilies count];
+
+			m_FontDictionary = [[NSMutableDictionary alloc] initWithCapacity:family_count];
+
+			for (unsigned ff = 0; ff < family_count; ff++)
+				{
+					NSString * family_name = (NSString *) [pAvailableFontFamilies objectAtIndex:ff];
+
+					NSArray * font_members = [[NSFontManager sharedFontManager] availableMembersOfFontFamily:family_name];
+
+					unsigned member_count = [font_members count];
+
+					for (unsigned fm = 0; fm < member_count; fm++)
+						{
+							NSArray * font = (NSArray *) [font_members objectAtIndex:fm];
+
+							NSString * name = (NSString *) [font objectAtIndex:0];
+
+							[m_FontDictionary setObject:family_name forKey:name];
+						}
+				}
+		}
+	if (m_FontDictionary && fontName)
+		{
+			familyName = (NSString *) [m_FontDictionary objectForKey:fontName];
+		}
+	return familyName;
+}
+
+- (void)appendPluginMenuItem:(NSMenuItem *)menuItem
+{
+	if (![m_PluginsTools containsObject:menuItem])
+		{
+			if ([m_PluginsTools count] == 0)
+				{
+					[m_AppMenu[XAP_CocoaAppMenu_Tools] addItem:m_PluginsToolsSeparator];
+				}
+			[m_AppMenu[XAP_CocoaAppMenu_Tools] addItem:menuItem];
+
+			[m_PluginsTools addObject:menuItem];
+		}
+}
+
+- (void)removePluginMenuItem:(NSMenuItem *)menuItem
+{
+	if ([m_PluginsTools containsObject:menuItem])
+		{
+			[m_AppMenu[XAP_CocoaAppMenu_Tools] removeItem:menuItem];
+
+			if ([m_PluginsTools count] == 1)
+				{
+					[m_AppMenu[XAP_CocoaAppMenu_Tools] removeItem:m_PluginsToolsSeparator];
+				}
+			[m_PluginsTools removeObject:menuItem];
+		}
+}
+
 /* Do we need this? getLastFocussedFrame() should be tracking this now... [TODO!!]
  */
 - (void)setCurrentView:(AV_View *)view inFrame:(XAP_Frame *)frame
@@ -679,9 +847,16 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 	m_pViewCurrent  = view;
 	m_pFrameCurrent = frame;
 
-	if ([XAP_CocoaToolPalette instantiated])
+	[self notifyFrameViewChange];
+}
+
+- (void)resetCurrentView:(AV_View *)view inFrame:(XAP_Frame *)frame
+{
+	// UT_DEBUGMSG(("XAP_CocoaAppController - (void)resetCurrentView:(AV_View *)view inFrame:(XAP_Frame *)frame\n"));
+	if (m_pFrameCurrent == frame)
 		{
-			[[XAP_CocoaToolPalette instance:self] setCurrentView:view inFrame:frame];
+			m_pViewCurrent = view;
+			[self notifyFrameViewChange];
 		}
 }
 
@@ -703,6 +878,7 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 			m_pViewPrevious = 0;
 			m_pFramePrevious = 0;
 		}
+	[self notifyFrameViewChange];
 }
 
 - (AV_View *)currentView
@@ -725,4 +901,118 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 	return m_pFramePrevious;
 }
 
+- (void)notifyFrameViewChange
+{
+	if ([XAP_CocoaToolPalette instantiated])
+		{
+			[[XAP_CocoaToolPalette instance:self] setCurrentView:m_pViewCurrent inFrame:m_pFrameCurrent];
+		}
+
+	unsigned count = [m_Plugins count];
+
+	for (unsigned i = 0; i < count; i++)
+		{
+			XAP_CocoaPlugin * plugin = (XAP_CocoaPlugin *) [m_Plugins objectAtIndex:i];
+			[[plugin delegate] pluginCurrentDocumentHasChanged];
+		}
+}
+
+/* load .Abi bundle plugin at path, returns nil on failure
+ */
+- (XAP_CocoaPlugin *)loadPlugin:(NSString *)path
+{
+	if (!path)
+		return nil;
+
+	XAP_CocoaPlugin * cocoa_plugin = [[XAP_CocoaPlugin alloc] init];
+	if (!cocoa_plugin)
+		return nil;
+
+	if ([cocoa_plugin loadBundleWithPath:path])
+		{
+			[m_Plugins addObject:cocoa_plugin];
+		}
+	else
+		{
+			[cocoa_plugin release];
+			cocoa_plugin = nil;
+		}
+	return cocoa_plugin;
+}
+
+/* list of currently loaded plugins
+ */
+- (NSArray *)plugins
+{
+	return m_Plugins;
+}
+
+/* checks to see whether the plugins can deactivate, and, if they can, deactivates them;
+ * returns false if any of the plugins object
+ */
+- (BOOL)deactivateAllPlugins
+{
+	unsigned count = [m_Plugins count];
+
+	BOOL bCanDeactivate = YES;
+
+	for (unsigned i = 0; i < count; i++)
+		{
+			XAP_CocoaPlugin * plugin = (XAP_CocoaPlugin *) [m_Plugins objectAtIndex:i];
+
+			bCanDeactivate = [[plugin delegate] pluginCanDeactivate];
+			if (!bCanDeactivate)
+				break;
+		}
+	if (!bCanDeactivate)
+		return NO;
+
+	for (unsigned i = 0; i < count; i++)
+		{
+			XAP_CocoaPlugin * plugin = (XAP_CocoaPlugin *) [m_Plugins objectAtIndex:i];
+
+			[[plugin delegate] pluginDeactivate];
+		}
+	return YES;
+}
+
+/* checks to see whether the plugins can deactivate, and, if they can, deactivates them;
+ * returns false if the plugin objects, unless override is YES.
+ */
+- (BOOL)deactivatePlugin:(XAP_CocoaPlugin *)plugin overridePlugin:(BOOL)override
+{
+	if (!override)
+		{
+			if (![[plugin delegate] pluginCanDeactivate])
+				return NO;
+		}
+	[[plugin delegate] pluginDeactivate];
+
+	return YES;
+}
+
+/* This provides a mechanism for associating XAP_CocoaPlugin_MenuItem objects
+ * with a given menu ID.
+ */
+- (void)addRef:(AP_CocoaPlugin_MenuIDRef *)ref forMenuID:(NSNumber *)menuid
+{
+	[m_MenuIDRefDictionary setObject:ref forKey:menuid];
+}
+
+/* This provides a mechanism for finding XAP_CocoaPlugin_MenuItem objects associated
+ * with a given menu ID.
+ */
+- (AP_CocoaPlugin_MenuIDRef *)refForMenuID:(NSNumber *)menuid
+{
+	return (AP_CocoaPlugin_MenuIDRef *) [m_MenuIDRefDictionary objectForKey:menuid];
+}
+
+/* This provides a mechanism for removing XAP_CocoaPlugin_MenuItem objects associated
+ * with a given menu ID.
+ */
+- (void)removeRefForMenuID:(NSNumber *)menuid
+{
+	[m_MenuIDRefDictionary removeObjectForKey:menuid];
+}
+
 @end
